/***************************************************************************
 *   Copyright (C) 2007 by Bartosz Hoffmann   *
 *   monocrystal@o2.pl   *
 *                                                                         *
 ***************************************************************************/
#include "poldekinterface.h"
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
PoldekInterface::PoldekInterface( QObject *parent)
     : QProcess(parent)
{
	connect ( this , SIGNAL ( readyReadStandardOutput () ) , this , SLOT ( readStdio() ));

	//Tak wyglada wyjsciowy opis z poldka Lista jest uzywana do parsownania prosze nie zmienia
	//co prawda nie jest systematycznie utrzymany wie napawa to problemw
	header	<< "Summary:"
			<<"Group:"
			<< "Vendor:"
			<< "License:"
			<< "Arch/OS:"
			<< "URL:"
			<<"Built:"
			<<"Size:"
			<< "Package size:"
			<< "Path:"
			<< "File:"
			<< "Epoch:"
			<< "Description:";

	// A to jest nagwek formatujcy RPM eby lista wyjsciowa wygldaa jak ppowyzej :-)
	rpmHead	<< "\nPackage:%{NAME}"
				<< "Summary:%{SUMMARY}"
				<< "Group:%{GROUP}"
				<< "Vendor:%{VENDOR}"
				<< "License:%{LICENSE}"
				<< "Arch/OS:%{ARCH}/%{OS}"
				<< "URL:%{URL}"
				<< "Built:%{BUILDTIME}\t%{BUILDHOST}"
				<< "Size:%{SIZE}"
				<< "Package size:%{ARCHIVESIZE}"
				<< "Path:%{SOURCE}"
				<< "File:%{NAME}-%{VERSION}-%{EPOCH}.%{ARCH}.rpm"
				<< "Epoch:%{EPOCH}"
				<< "Description:%{DESCRIPTION}";
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
funkcja odpalajca POLDKA 
*/
void PoldekInterface::runPoldek ( QStringList arguments ){
	connect ( this , SIGNAL (  finished ( int , QProcess::ExitStatus  )) , this , SLOT ( poldekClosed (int , QProcess::ExitStatus) ));
	while ( waitForFinished ( ) );
	start ( "/usr/bin/poldek" , arguments );
	emit allData( 0 );
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
funkcja odpalajca w X-TERMINAL-u program jaki sobie zayczymy i podamy go w argumentacji 
*/
void PoldekInterface::runXtermCommand ( QStringList arguments ){
	connect ( this , SIGNAL (  finished ( int , QProcess::ExitStatus  )) , this , SLOT ( xtermClosed (int , QProcess::ExitStatus) ));
	while ( waitForFinished ( ) );
	start( "/usr/X11R6/bin/xterm" ,   QStringList()   <<  "-bg" << "black"
												<< "-fg" << "green"
												<< "-e" << arguments );
	emit allData( 0 );
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
funkcja odpalajca RPM
*/
void PoldekInterface::runRPM(  QStringList arguments ){
	connect ( this , SIGNAL (  finished ( int , QProcess::ExitStatus  )) , this , SLOT ( rpmClosed(int , QProcess::ExitStatus) ));
	while ( waitForFinished ( ) );
	start ( "/bin/rpm" , arguments );
	emit allData( 0 );
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//odczyt z wejscia standartowego :-) z dekodoawaniem tekstu z iso-8858-2 na unicode
void PoldekInterface::readStdio (){
	QTextCodec *codec = QTextCodec::codecForName("ISO 8859-2");
 	QTextDecoder *decoder = codec->makeDecoder();
	tmp_list += decoder->toUnicode(readAll());
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/** Te trzy cda poniej to funkcje,
 * ktre mog wysya sygnay
 * w razie gdyby sie co wykrzaczyo */

void PoldekInterface::poldekClosed ( int exitCode, QProcess::ExitStatus exitStatus ){
	if ( exitStatus == QProcess::NormalExit ){
		emit poldekFinished();
	}
	else if ( exitStatus ==QProcess::CrashExit  ){
		emit poldekCrash();
	}
	disconnect ( this , SIGNAL (  finished ( int , QProcess::ExitStatus  )) , this , SLOT ( poldekClosed (int , QProcess::ExitStatus) ));
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PoldekInterface::xtermClosed ( int exitCode, QProcess::ExitStatus exitStatus ){
	if ( exitStatus == QProcess::NormalExit ){
		emit xtermFinished( exitCode );
	}
	else if ( exitStatus ==QProcess::CrashExit  ){
		emit xtermCrash( exitCode );
	}
	disconnect ( this , SIGNAL (  finished ( int , QProcess::ExitStatus  )) , this , SLOT ( xtermClosed (int , QProcess::ExitStatus) ));
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PoldekInterface::rpmClosed ( int exitCode, QProcess::ExitStatus exitStatus ){
	if ( exitStatus == QProcess::NormalExit ){
		emit rpmFinished();
	}
	else if ( exitStatus ==QProcess::CrashExit  ){
		emit rpmCrash();
	}
	disconnect ( this , SIGNAL (  finished ( int , QProcess::ExitStatus  )) , this , SLOT ( rpmClosed (int , QProcess::ExitStatus) ));
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Funkcja ktra pobiera list pakietw zainstalowanych poprzez wywoanie RPM
void PoldekInterface::InstalledList (){
	connect ( this , SIGNAL ( rpmFinished() ) , this ,SLOT( makeInstaledList( )  ) );
	runRPM( QStringList()  << "--queryformat"  << rpmHead.join("\n")  <<  "-qa" );
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Analogiczna do poprzedniej tylko pobiera wszystkie zainstalowane i niezainstalowane z PolDka
void PoldekInterface::PkgList (){
	connect ( this , SIGNAL ( poldekFinished() ) , this , SLOT ( makePkgList() ) );
	runPoldek ( QStringList() << "-q" << "--shcmd" << "desc *" );
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Parser Zainstalowanych rozdziela pakiety w Mapy
void PoldekInterface:: makeInstaledList( ) {
	installedPkg <<  tmp_list.split( "\n" , QString::SkipEmptyParts );
	emit allData( installedPkg.count() );
	int count =0;
	instaledMap.clear();
		for (QStringList::Iterator it = installedPkg.begin()  ;  it != installedPkg.end(); ++it  , ++count){
			if ( QString(*it).contains( "Package:" ))
			{
				QString tmp_text = QString(*it).remove("Package:");
				QMap<QString , QString> tmp_map;
				it++;
				count++;
				for (QStringList::Iterator iter = header.begin()  ;  iter != header.end(); ++iter , it++ , count++)
				{
					if ( !QString(*it).contains("Description:") )
						{
							tmp_map[*iter] = QString(*it).remove(*iter).trimmed();
						}
					else
						{
							QString description;
							while ( true )
								{
									description += QString(*it).remove(*iter)+" ";
									count++;
									if (  ++it == installedPkg.end() ) { it-=2; break; }
									if (  QString(*it).contains("Package:") ) { it-=2; break;}
								}
							tmp_map[*iter] = description;
						}
				instaledMap[tmp_text] = tmp_map;
				}
			}
			emit dataRead( count );
		}
	emit dataRead( installedPkg.count() );
	tmp_list.clear();
	installedPkg.clear();
	disconnect ( this , SIGNAL ( rpmFinished() ) , this , SLOT ( makeInstaledList() ) );
	emit InstaledListDone();
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Parser niezainstalowanych rozdziela pakiety w Mapy
void PoldekInterface:: makePkgList() {
	allMap.clear();
	allMap = parser ( tmp_list.split( "\n" ,  QString::SkipEmptyParts ) );
	tmp_list.clear();
	disconnect ( this , SIGNAL ( poldekFinished() ) , this , SLOT ( makePkgList( ) ) );
	emit AllListDone();
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString PoldekInterface::getInstalledValue( QString key , QString second_key ){
	return instaledMap.value(key).value(second_key);
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString PoldekInterface::getAllValue( QString key , QString second_key ){
	return allMap.value(key).value(second_key);
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QStringList PoldekInterface::getInstalledKeys(){
	return instaledMap.keys();
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QStringList PoldekInterface::getAllKeys(){
	return allMap.keys();
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QStringList PoldekInterface::getAllGroups() {
	return pkgGroup;
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PoldekInterface::moveToNotInstalled ( QStringList keys ) {
	for ( QStringList::Iterator it = keys.begin() ; it != keys.end() ; ++it )
	{
		allMap.insert( *it , instaledMap.take( *it ) );
	}
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QMap < QString , QMap<QString , QString > >  PoldekInterface::parser ( QStringList lista )
{
	int count=0;
	emit allData( lista.count() );
	QMap < QString , QMap<QString , QString > > mapa_pakietow;
		for (QStringList::Iterator it =lista.begin()  ;  it != lista.end(); it++ , count++ ){
			if ( QString(*it).contains( "Package:" ))
			{
				QString tmp_text = QString(*it).remove("Package:").trimmed();
				tmp_text = tmp_text.remove(tmp_text.section( '-' , tmp_text.count( '-' )-1 ,  tmp_text.count( '-' )  ));
				tmp_text = tmp_text.remove( tmp_text.count()-1 , 1);
				QMap<QString , QString> tmp_map;
				it++;
				count++;
				for (QStringList::Iterator iter = header.begin()  ;  iter != header.end();  iter++ )
					{
						if ( QString(*it).contains(*iter) && !QString(*it).contains("Description:") )
						{
							tmp_map[*iter] = QString(*it++).remove(*iter).trimmed();
							++count;
						}
						else if ( QString(*it).contains("Description:") && QString(*iter).contains ("Description:") )
						{
							QString description;
							while ( true )
							{
								if (  QString(*it).contains("Package:") ) { it-=2;  break;}
								description += QString(*it).remove(*iter).trimmed()+ " ";
								++count;
								if (  ++it == lista.end() ) { it--;  break; }
							}
							tmp_map[*iter] = description;
						}
					}
				mapa_pakietow[tmp_text] = tmp_map;
			}
			emit dataRead( count );
		}
	emit dataRead( lista.count() );
	return mapa_pakietow;
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////